%
%  Graphing of tracks from a common origin.
%
%  Copyright PBeemiller 2013.
%
%  To make this tool available to Imaris, copy this file into the XTensions
%  folder in the Imaris installation folder, e.g., C:\Program
%  Files\Bitplane\Imaris x64 7.6.1\XTensions. After copying this file to
%  the XTensions folder and restarting Imaris, you can find this function
%  in the Surfaces Function menu, as well as in the Image Processing menu
%  under the Spots and Surfaces Functions groups.
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Graph tracks with a common origin" icon="Matlab"
%        tooltip="Plot tracks from a common origin.">
%          <Command>MatlabXT::plottrackscommonorigin(%i)</Command>
%        </Item>
%       </Submenu>
%       <Submenu name="Spots Functions">
%        <Item name="Graph tracks with a common origin" icon="Matlab"
%        tooltip="Plot tracks from a common origin.">
%          <Command>MatlabXT::plottrackscommonorigin(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Graph tracks with a common origin" icon="Matlab"
%          tooltip="Plot tracks from a common origin.">
%            <Command>MatlabXT::plottrackscommonorigin(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSpots">
%          <Item name="Graph tracks with a common origin" icon="Matlab"
%          tooltip="Plot tracks from a common origin.">
%            <Command>MatlabXT::plottrackscommonorigin(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%  Description: Plots spots or surfaces tracks from a common origin.
% 
%  Copyright 2013 - P. Beemiller. Licensed under a Creative Commmons
%  Attribution license. Please see: http://creativecommons.org/licenses/by/3.0/


function plottrackscommonorigin( xImarisID )
    % PLOTTRACKSCOMMONORIGIN Plot Surpass tracks with a common origin.
    %   PLOTTRACKSCOMMONORIGIN generates a graph of all tracks, with tracks
    %   shifted to have an origin at (0, 0, 0).
    %   
    %   The only input is the ID of the Imaris instance with the object
    %   tracks to be graphed. The function will create a list of the Spots
    %   and Surfaces present in the Surpass scene and prompt the user to
    %   select the object's tracks to plots. The resulting graph can be
    %   re-scaled, rotated, etc., then output as an image or vector art for
    %   presentation.
    
    %% Get the Imaris instance ID.
    % If the extension is compiled, the Imaris ID will be passed as a string.
    if ischar(xImarisID)
        xImarisID = str2double(xImarisID);
    end
    
    %% Append ImarisLib to the dynamic path. 
    if isempty(javaclasspath('-dynamic')) || ...
            cellfun(@isempty, regexp(javaclasspath('-dynamic'), 'ImarisLib.jar', 'Start', 'Once'))
        javaaddpath '.\ImarisLib.jar'
    end
    
    %% Create a new object from the ImarisLib class.
    xImarisLib = ImarisLib;

    %% Connect to the calling Imaris instance.
    xImarisApp = xImarisLib.GetApplication(xImarisID);
    
    %% Get the Surpass objects.
    surpassObjects = getsporfaces(xImarisApp, 'Both');
    
    %% Prompt the user for the Surpass object to replot.
    listIdx = listdlg('ListString', {surpassObjects.Name}, 'ListSize', [160 200], ...
        'Name', 'Object Selection', 'SelectionMode', 'single', ...
        'PromptString', 'Select object to graph:');

    %% Get the Surpass object.
    surpassSelection = surpassObjects(listIdx).ImarisObject;

    %% Get the Surpass object data.
    if xImarisApp.GetFactory.IsSpots(surpassSelection)

        %% Do the Spots-specific XT calls.

        % Cast the object to Spots.
        surpassObject = xImarisApp.GetFactory.ToSpots(surpassSelection);

        % Get the spot positions.
        objectPos = surpassObject.GetPositionsXYZ;

    else % xImarisApp.GetFactory.IsSurfaces(surpassSelection)

        %% Do the Surfaces-specific XT calls.

        % Cast the object to Surfaces.
        surpassObject = xImarisApp.GetFactory.ToSurfaces(surpassSelection);

        % Get the number of surfaces.
        surfaceCount = surpassObject.GetNumberOfSurfaces;

        % Get the surface positions and times.
        objectPos = zeros(surfaceCount, 3);
        for s = 1:surfaceCount
            objectPos(s, :) = surpassObject.GetCenterOfMass(s - 1);
        end % s

    end % if
    
    %% Allocate an array to collect all the recentered positions.
    centeredPos = zeros(size(objectPos));
    
    %% Get the track information.
    objectIDs = surpassObject.GetTrackIds;
    objectEdges = surpassObject.GetTrackEdges;
    trackLabels = unique(objectIDs);
    
    %% Create a figure for plotting.
    hFig = figure;
    hAxes = axes('Position', [0.13 0.11 0.775 0.775], 'Parent', hFig, ...
        'Linewidth', 2, 'TickDir', 'out', 'FontSize', 12);
    
    %% Translate all the tracks to a common origin and plot.
    for r = 1:length(trackLabels)
        % Get indices for the track.
        rEdges = objectEdges(objectIDs == trackLabels(r), :);
        rSpots = unique(rEdges);

        % Get the track positions.
        rPos = objectPos(rSpots + 1, :);
        
        % Shift the track coordinates to place the origin at zero.
        cPos = bsxfun(@minus, rPos, rPos(1, :));
        
        % Add the centered coordinates to the list of all track positions.
        centeredPos(rSpots + 1, :) = cPos;
        
        % Plot the shifted track.
        line(cPos(:, 1), cPos(:, 2), cPos(:, 3), 'LineWidth', 2) 
    end
    
    %% Make the axes symmetric about zero.
    % For x:
    xRange = xlim(hAxes);
    xExtreme = max(abs(xRange));
    xlim(hAxes, [-xExtreme xExtreme])
    
    % For y:
    yRange = ylim(hAxes);
    yExtreme = max(abs(yRange));
    ylim(hAxes, [-yExtreme yExtreme])
    
    % For z:
    zRange = zlim(hAxes);
    zExtreme = max(abs(zRange));
    zlim(hAxes, [-zExtreme zExtreme])
    
    %% Format the axes.
    axis(hAxes, 'equal')
    
    % Setup identical x and y ticks.
    xTicks = get(hAxes, 'XTick');
    xTickLabels = get(hAxes, 'XTickLabel');
    
    set(hAxes, 'YTick', xTicks, 'YTickLabel', xTickLabels)
    
    %% Add lines to serve as coordinate axes.
    xAx = line(xlim, [0 0], [0 0], 'Color', 'k', 'LineStyle', ':', 'LineWidth', 2);
    yAx = line([0 0], ylim, [0 0], 'Color', 'k', 'LineStyle', ':', 'LineWidth', 2);
    zAx = line([0 0], [0 0], zlim, 'Color', 'k', 'LineStyle', ':', 'LineWidth', 2);
    
    uistack(xAx, 'bottom')
    uistack(yAx, 'bottom')
    uistack(zAx, 'bottom')
    
    %% Label the axes with the Imaris units.
    imarisUnits = char(xImarisApp.GetDataSet.GetUnit);
    
    % If the units are in microns, add a slash for latex. I think Imaris
    % always uses microns, but just in case.
    if strcmp(imarisUnits, 'um')
        imarisUnits = '\mum';
    end
    
    xlabel(['x (' imarisUnits ')'])
    ylabel(['y (' imarisUnits ')'])
    zlabel(['z (' imarisUnits ')'])
    
    %% Add a menu with choices for doing axes adjustments.
    % Create the menu.    
    adjMenu = uimenu(hFig, 'Label', 'Axes');
    
    % Create the axes limit adjustment menu option.
    adjAxesMenuItem = uimenu(adjMenu, ...
        'Label', 'Adjust limits', ...
        'Callback', @(adjAxesMenuItem, hEvent)adjustaxeslimits(...
        adjAxesMenuItem, hEvent, hFig, hAxes, xAx, yAx, zAx));
    
    % Create the view (rotations) adjustment menu option.
    adjAxesAngleMenuItem = uimenu(adjMenu, ...
        'Label', 'Adjust view', ...
        'Callback', @(adjAxesMenuItem, hEvent)adjustaxesview(...
        adjAxesMenuItem, hEvent, hFig, hAxes));        
end % plottrackscommonorigin


function [ surpassObjects ] = getsporfaces( xImarisApp, varargin )
    % GETSPORFACES Get the Spots and/or Surfaces objects from Imaris.
    %   GETSPORFACES returns a struct containing the Spots and/or
    %   Surfaces child objects from the Surpass scene of the input Imaris XT
    %   application.
    %
    %   surpassObjects is a struct with the fields 'Name' and 'ImarisObject'.
    %   The Name field is the char string from the Imaris Surpass scene
    %   control. The ImarisObject field contains the IDataItemPrxHelper
    %   associated with the Surpass object.
    %
    %   The first input argument must be an IApplicationPrxHelper object for a
    %   running Imaris instance. The optional second input argument is a string
    %   'Spots', 'Surfaces' or 'Both' that indicates which objects the user
    %   wants included in the output. The function is not case sensitive.


    %% Parse the inputs.
    % The user can optionally leave out the types of objects to get. In that
    % case, we default to returning both Spots and Surfaces.
    if nargin == 1
        objectsToGet = 'Both';
        
    elseif nargin == 2
        objectsToGet = varargin{1};
        
    end % if

    %% Allocate the output struct.
    surpassObjects = struct('Name', {}, 'ImarisObject', []);
    
    %% Get the Surpass objects.
    % Create a factory handle.
    xFactory = xImarisApp.GetFactory;

    % Use a switch case to return either the Spots, Surfaces or both.
    switch lower(objectsToGet)

        case 'both'

            for m = 1:xImarisApp.GetSurpassScene.GetNumberOfChildren
                % Get the next child of the Surpass container.
                mChild = xImarisApp.GetSurpassScene.GetChild(m - 1);

                % If the child is a Surfaces or Spots object, add it to the list.
                if xFactory.IsSpots(mChild) || xFactory.IsSurfaces(mChild)
                    surpassObjects(length(surpassObjects) + 1).ImarisObject = mChild;
                    surpassObjects(length(surpassObjects)).Name = char(mChild.GetName);
                end % if
            end % for

        case 'spots'

            for m = 1:xImarisApp.GetSurpassScene.GetNumberOfChildren
                % Get the next child of the Surpass container.
                mChild = xImarisApp.GetSurpassScene.GetChild(m - 1);

                % If the child is a Spots object, add it to the list.
                if xFactory.IsSpots(mChild)
                    surpassObjects(length(surpassObjects) + 1).ImarisObject = mChild;
                    surpassObjects(length(surpassObjects)).Name = char(mChild.GetName);
                end % if
            end % for

        case 'surfaces'

            for m = 1:xImarisApp.GetSurpassScene.GetNumberOfChildren
                % Get the next child of the Surpass container.
                mChild = xImarisApp.GetSurpassScene.GetChild(m - 1);

                % If the child is a Surfaces object, add it to the list.
                if xFactory.IsSurfaces(mChild)
                    surpassObjects(length(surpassObjects) + 1).ImarisObject = mChild;
                    surpassObjects(length(surpassObjects)).Name = char(mChild.GetName);
                end % if
            end % for

    end % switch
end % getsporfaces


%% Axes adjustment functions:


function adjustaxeslimits(hMenu, hEvent, hFig, hAxes, xAx, yAx, zAx)
    
    %% Check for an existing limit adjustment figure.
    adjLimitsWindow = getappdata(hFig, 'adjLimitsWindow');
    
    % If the adjustment window exists, raise it and return.
    if ~isempty(adjLimitsWindow)
        figure(adjLimitsWindow)
        return
    end
    
    %% Create the adjustment figure.
    adjLimitsPos = zeros(1, 4);
    
    % Get the parent figure position.
    parentPosition = get(hFig, 'Position');
    
    % Get the center of the parent.
    adjLimitsPos(1) = parentPosition(1) + parentPosition(3)/2 - 125;
    adjLimitsPos(2) = parentPosition(2) + parentPosition(4)/2 - 100;
    
    % Size the figure.
    adjLimitsPos(3:4) = [250 200];
    
    % Create the figure.
    adjLimitsFig = figure('Position', adjLimitsPos, ...
        'Name', 'Adjust axes limits', 'NumberTitle', 'off', 'Resize', 'off');
    
    % Create an axes.
    adjLimitsAxes = axes('Parent', adjLimitsFig);
    
    % Hide the axes.
    axis(adjLimitsAxes, 'off')
    
    %% Create the labels.
    text(-20, 125, 'X Range', 'Units', 'Pixels');
    text(-20, 75, 'Y Range', 'Units', 'Pixels');
    text(-20, 25, 'Z Range', 'Units', 'Pixels');
    
    text(40, 155, 'Minimum', 'Units', 'Pixels');
    text(112, 155, 'Maximum', 'Units', 'Pixels');
    
    %% Create the adjustment edit boxes.
    % X adjustment boxes:
    xInitial = get(hAxes, 'XLim');
    xMinAdj = uicontrol('Style', 'edit', ...
        'Position', [75 135 48 24], ...
        'String', xInitial(1), ...
        'Tag', 'xminedit', ...
        'Callback', @xrangechange);
    
    xMaxAdj = uicontrol('Style', 'edit', ...
        'Position', [150 135 48 24], ...
        'String', xInitial(2), ...
        'Tag', 'xmaxedit', ...
        'Callback', @xrangechange);
    
    % Y adjustment boxes:
    yInitial = get(hAxes, 'XLim');
    yMinAdj = uicontrol('Style', 'edit', ...
        'Position', [75 85 48 24], ...
        'String', yInitial(1), ...
        'Tag', 'yminedit', ...
        'Callback', @yrangechange);
    
    yMaxAdj = uicontrol('Style', 'edit', ...
        'Position', [150 85 48 24], ...
        'String', yInitial(2), ...
        'Tag', 'ymaxedit', ...
        'Callback', @yrangechange);
    
    % Z adjustment boxes:
    zInitial = get(hAxes, 'XLim');
    zMinAdj = uicontrol('Style', 'edit', ...
        'Position', [75 35 48 24], ...
        'String', zInitial(1), ...
        'Tag', 'zminedit', ...
        'Callback', @zrangechange);
    
    zMaxAdj = uicontrol('Style', 'edit', ...
        'Position', [150 35 48 24], ...
        'String', zInitial(2), ...
        'Tag', 'zmaxedit', ...
        'Callback', @zrangechange);
    
    %% Remove the adjustment figure menu bar.
    set(adjLimitsFig, 'MenuBar', 'none')
    
    %% Store the adjustment figure's handle in the appdata of the main figure.
    setappdata(hFig, 'adjLimitsWindow', adjLimitsFig)
    
    %% Update the figure close functions.
    set(hFig, 'CloseRequestFcn', @(hFig, hEvent)hfigclosereqfcn(hFig, hEvent))
    set(adjLimitsFig, 'CloseRequestFcn', @(adjLimitsFig, hEvent)adjlimitsfigclosereqfcn...
        (adjLimitsFig, hEvent, hFig))
    
    
    %% Nested functions for axes limit adjustments:

    
    function xrangechange(varargin)
        %% Get the current axes x limits.
        currentXLim = get(hAxes, 'XLim');
        
        %% Get the desired range.
        xMin = str2double(get(xMinAdj, 'String'));
        xMax = str2double(get(xMaxAdj, 'String'));
        
        %% Test for a valid range, then update or reset.
        if xMin >= xMax
            % Reset the x editbox values.
            set(xMinAdj, 'String', currentXLim(1))
            set(xMaxAdj, 'String', currentXLim(2))

        else
            % Update the axes range.
            set(hAxes, 'XLim', [xMin xMax])
            
            % Update the xdata for the x axes center line.
            set(xAx, 'XData', [xMin xMax])
            
        end % if
    end % xrangechange
    
    
    function yrangechange(varargin)
        %% Get the current axes x limits.
        currentYLim = get(hAxes, 'YLim');
        
        %% Get the desired range.
        yMin = str2double(get(yMinAdj, 'String'));
        yMax = str2double(get(yMaxAdj, 'String'));
        
        %% Test for a valid range, then update or reset.
        if yMin >= yMax
            % Reset the y editbox values.
            set(yMinAdj, 'String', currentYLim(1))
            set(yMaxAdj, 'String', currentYLim(2))

        else
            % Update the axes range.
            set(hAxes, 'YLim', [yMin yMax])
            
            % Update the xdata for the x axes center line.
            set(yAx, 'YData', [yMin yMax])
            
        end % if
    end % xrangechange
    
    
    function zrangechange(varargin)
        %% Get the current axes x limits.
        currentZLim = get(hAxes, 'ZLim');
        
        %% Get the desired range.
        zMin = str2double(get(zMinAdj, 'String'));
        zMax = str2double(get(zMaxAdj, 'String'));
        
        %% Test for a valid range, then update or reset.
        if zMin >= zMax
            % Reset the z editbox values.
            set(zMinAdj, 'String', currentZLim(1))
            set(zMaxAdj, 'String', currentZLim(2))

        else
            % Update the axes range.
            set(hAxes, 'ZLim', [zMin zMax])
            
            % Update the xdata for the x axes center line.
            set(zAx, 'ZData', [zMin zMax])
            
        end % if
    end % xrangechange
end % adjustaxes


function adjustaxesview(hMenu, hEvent, hFig, hAxes)
    
    %% Check for an existing view adjustment figure.
    adjViewWindow = getappdata(hFig, 'adjViewWindow');
    
    % If the adjustment window exists, raise it and return.
    if ~isempty(adjViewWindow)
        figure(adjViewWindow)
        return
    end
    
    %% Create the adjustment figure.
    adjViewPos = zeros(1, 4);
    
    % Get the parent figure position.
    parentPosition = get(hFig, 'Position');
    
    % Get the center of the parent.
    adjViewPos(1) = parentPosition(1) + parentPosition(3)/2 - 75;
    adjViewPos(2) = parentPosition(2) + parentPosition(4)/2 - 75;
    
    % Size the figure.
    adjViewPos(3:4) = [150 150];
    
    % Create the figure.
    adjViewFig = figure('Position', adjViewPos, ...
        'Name', 'Adjust axes view', 'NumberTitle', 'off', 'Resize', 'off');
    
    % Create an axes.
    adjViewAxes = axes('Parent', adjViewFig);
    
    % Hide the axes.
    axis(adjViewAxes, 'off')
    
    %% Create the angle and value labels.
    text(-6, 81, 'Azimuth', 'Units', 'Pixels');
    text(-6, 31, 'Elevation', 'Units', 'Pixels');
    text(53, 107, 'Degrees', 'Units', 'Pixels');
    
    %% Create the view adjustment edit boxes.
    % Get the initial view.
    viewInitial = get(hAxes, 'View');
    
    % Azimuth adjustment box:
    azimuthAdj = uicontrol('Style', 'edit', ...
        'Position', [75 85 48 24], ...
        'String', viewInitial(1), ...
        'Tag', 'azimuthedit', ...
        'Callback', @azimuthchange);
    
    % Elevation adjustment box:
    elevationAdj = uicontrol('Style', 'edit', ...
        'Position', [75 35 48 24], ...
        'String', viewInitial(2), ...
        'Tag', 'elevationedit', ...
        'Callback', @elevationchange);
       
    %% Remove the adjustment figure menu bar.
    set(adjViewFig, 'MenuBar', 'none')
    
    %% Store the adjustment figure's handle in the appdata of the main figure.
    setappdata(hFig, 'adjViewWindow', adjViewFig)
    
    %% Attach a listener to the axes to update the azimuth and elevation on a view change.
    viewListener = addlistener(hAxes, 'View', 'PostSet', ...
        @(hAxes, hEvent)viewsync(hAxes, hEvent, azimuthAdj, elevationAdj));
    
    %% Update the figure close functions.
    set(hFig, 'CloseRequestFcn', @(hFig, hEvent)hfigclosereqfcn(hFig, hEvent))
    set(adjViewFig, 'CloseRequestFcn', @(adjViewFig, hEvent)adjviewfigclosereqfcn...
        (adjViewFig, hEvent, hFig, viewListener))
    
    
    %% Nested functions for axes limit adjustments:

    
    function azimuthchange(varargin)
        %% Get the current axes x limits.
        currentView = get(hAxes, 'View');
        
        %% Get the desired azimuth value.
        azValue = str2double(get(azimuthAdj, 'String'));
        
        %% Test for a valid value, then update or reset.
        if ~isreal(azValue)
            % Reset the azimuth editbox value.
            set(azimuthAdj, 'String', currentView(1))

        else
            % Update the axes view.
            set(hAxes, 'View', [azValue currentView(2)])
                        
        end % if
    end % azimuthchange
    
    
    function elevationchange(varargin)
        %% Get the current axes x limits.
        currentView = get(hAxes, 'View');
        
        %% Get the elevation value.
        elValue = str2double(get(elevationAdj, 'String'));
        
        %% Test for a valid range, then update or reset.
        if ~isreal(elValue)
            % Reset the elevation editbox value.
            set(elevationAdj, 'String', currentView(2))

        else
            % Update the axes view.
            set(hAxes, 'View', [currentView(1) elValue])
            
        end % if

    end % elevationchange
end % adjustaxesview


%% Sync functions for the adjustment figures:


function viewsync(hSrc, hEvent, azimuthAdj, elevationAdj)
    
    if ishandle(azimuthAdj)
        %% Get the view.
        viewValue = hEvent.NewValue;

        %% Set the azimuth and elevation edit boxes.
        set(azimuthAdj, 'String', viewValue(1))
        set(elevationAdj, 'String', viewValue(2))
    end
end % viewsync


%% Close request functions:


function hfigclosereqfcn(hFig, hEvent)
    
    %% Check for associated axes adjustment figures.
    adjLimitsWindow = getappdata(hFig, 'adjLimitsWindow');
    adjViewWindow = getappdata(hFig, 'adjViewWindow');
    
    %% Delete any adjustment figures.
    if ~isempty(adjLimitsWindow)
        delete(adjLimitsWindow)
    end
    
    if ~isempty(adjViewWindow)
        delete(adjViewWindow)
    end
    
    %% Delete the figure.
    delete(hFig)
end % hfigclosereqfcn


function adjlimitsfigclosereqfcn(adjLimitsFig, hEvent, hFig)
    
    %% Remove the adjustment figure handle from the main figure's appdata.
    setappdata(hFig, 'adjLimitsWindow', [])
    
    %% Delete the adjustment figure.
    delete(adjLimitsFig)
end % adjlimitsfigclosereqfcn


function adjviewfigclosereqfcn(adjViewFig, hEvent, hFig, viewListener)
    
    %% Remove the adjustment figure handle from the main figure's appdata.
    setappdata(hFig, 'adjViewWindow', [])
    
    %% Delete the view sync listener.
    delete(viewListener)
    
    %% Delete the adjustment figure.
    delete(adjViewFig)

end % adjviewfigclosereqfcn

